/*
   Hyllian's DDT Shader
   
   Copyright (C) 2011-2016 Hyllian/Jararaca - sergiogdb@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/ 


#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif

uniform sampler2D sampler0;
varying vec2 v_position;
uniform vec2 u_texelDelta;
uniform vec2 u_pixelDelta;

const vec2 size = vec2(480.0, 272.0);
const vec2 dx = vec2(0.00208333, 0.0);
const vec2 dy = vec2(0.0, 0.00367647);

const vec3 Y = vec3(0.2126, 0.7152, 0.0722);

float luma(vec3 color)
{
  return dot(color, Y);
}

vec3 bilinear(float p, float q, vec3 A, vec3 B, vec3 C, vec3 D)
{
	return ((1.0-p)*(1.0-q)*A + p*(1.0-q)*B + (1.0-p)*q*C + p*q*D);
} 

vec3 texture2dd (vec2 texCoord) {
	
	vec2 loc = texCoord*size;
	vec2 pc  = (floor(loc) + vec2(0.5,0.5))/size;
	
	vec2 pos = fract(loc)-vec2(0.5, 0.5); // pos = pixel position
	vec2 dir = sign(pos); // dir = pixel direction

	vec2 g1 = dir*dx;
	vec2 g2 = dir*dy;

	vec3 A = texture2D(sampler0, pc       ).xyz;
	vec3 B = texture2D(sampler0, pc +g1   ).xyz;
	vec3 C = texture2D(sampler0, pc    +g2).xyz;
	vec3 D = texture2D(sampler0, pc +g1+g2).xyz;

	float a = luma(A);
	float b = luma(B);
	float c = luma(C);
	float d = luma(D);

	float p = abs(pos.x);
	float q = abs(pos.y);

	float k = distance(pos,g1);
	float l = distance(pos,g2);

	float wd1 = abs(a-d);
	float wd2 = abs(b-c);

	if ( wd1 < wd2 )
	{
		if (k < l)
		{
			C = A + D - B;
		}
		else
		{
			B = A + D - C;
		}
	}
	else if (wd1 > wd2)
	{
		D = B + C - A;
	}

	return bilinear(p, q, A, B, C, D);
}

void main() {
	
	gl_FragColor = vec4(texture2dd(v_position+vec2(0.0000001, 0.0000001)),1.0);
}